% fft_params.{fs, band_boundaries, window_size, overlapping_windows}
% search_params.{window_size (in fft coef), window_step}
% assume 16 bands
%   01 - 16: band_mean_t - mean power of each band over time
%   17 - 32: band_std_t - std power of each band over time
%   33 - 34: mpb - mean most powerful band of 4 , most powerful band of 16           
%   35:      mpb_std_t - std of most powerful band of 16
%   36:      band_width_50 - width of smallest set of bands containing 50 of power
%   37:      nsb_mean_t - mean of number of bands with greater than 25% power
%   38:      nsb_std_t - std of number of bands with greater than 25% power
%   39:      power_std - std of total power over time
%   40:      power_peaks - number of power peaks (below 30% to above 70%)
%   41:      periodicity - regularity of power peaks
%   42:      power range
%   43:      mean power
%   44 - 47: band_mean_t - reduced to 4 bands
%   48 - 51: band_std_t - reduced to 4 bands
%   52 - 138: time-dependent frequency information at different resolutions
function [features, time_locations, data_locations] = ...
            extract_global_features_track(data, fft_params, search_params)


    % compute the wfft
    % fft_data(num_bands, num_times)
    all_fft_data = compute_windowed_fft(data, fft_params.fs, fft_params.band_boundaries, ...
        fft_params.window_size, fft_params.overlapping_windows);
    
    all_fft_pow = sum(all_fft_data, 1);
    for i = 1:size(all_fft_data, 2)
        if all_fft_pow(i) ~= 0
            all_fft_data(:, i) = all_fft_data(:, i) / all_fft_pow(i);            
        end        
    end
    all_coarse_fft_data = zeros(4, size(all_fft_data, 2));
    for i = 1:4
        all_coarse_fft_data(i, :) = sum(all_fft_data((i-1)*4+1:(i-1)*4+4, :), 1);
    end
    
    track_length = size(all_fft_data, 2);    
    num_windows = floor((track_length-search_params.window_size)/search_params.window_step);
    %disp(['num windows ' num2str(num_windows)]);
    
    fft_params.window_step = fft_params.window_size/fft_params.overlapping_windows;
    
    window_width_time = fft_params.window_size + fft_params.window_step*(search_params.window_size-1);
    first_window_time = window_width_time/2/fft_params.fs;            
    time_per_window = fft_params.window_step*search_params.window_step/fft_params.fs;
    
    features = zeros(138, num_windows);
    time_locations = zeros(num_windows, 1);
    data_locations = zeros(num_windows, 2);
    
    for w = 1:num_windows
        i1 = (w-1)*search_params.window_step+1;
        i2 = i1 + search_params.window_size - 1;
        if w == 1
            disp(num2str(i2-i1+1))
        end
            
        fft_data = all_fft_data(:, i1:i2);
        if max(max(fft_data)) == 0
            disp('warning zero data');
        end
        coarse_fft_data = all_coarse_fft_data(:, i1:i2);                
        fft_pow = all_fft_pow(i1:i2);
        time_locations(w) = first_window_time + (w-1)*time_per_window;
        data_locations(w, :) = floor([(i1-1)*fft_params.window_step+1  ...
                             (i2-1)*fft_params.window_step+fft_params.window_size]);
        
        % mean and std power
        features(1:16, w) = mean(fft_data, 2);
        features(17:32, w) = std(fft_data, 0, 2);

        % most powerful band (perhaps should change mean to mode)
        [tmp, mpb_16] = max(fft_data, [], 1);
        mpb_4 = mpb_16/4;
        features(33, w) = ceil(mean(mpb_4));
        features(34, w) = round(mean(mpb_16));
        features(35, w) = std(mpb_16);

        % band width
        band_pow = sum(fft_data, 2);
        total_pow = sum(band_pow);
        found = 0;
        for width = 1:16
            for b = 1:16-width+1
                if sum(band_pow(b:b+width-1)/total_pow) > 0.5
                    found = 1;                
                    break;
                end
            end
            if found
                features(36) = width;
                break;
            end
        end

        % bands over 25% power
        num_bands_over_25 = sum(fft_data > 0.25, 1);
        features(37, w) = mean(num_bands_over_25);
        features(38, w) = std(num_bands_over_25);

        % std power
        features(39, w) = std(fft_pow);

        % power peaks
        min_power = min(fft_pow);
        max_power = max(fft_pow);
        power_70 = 0.7*(max_power-min_power)+min_power;
        power_30 = 0.3*(max_power-min_power)+min_power;    
        had_low = 1;
        had_high = 0;        
        for t = 1:length(fft_pow)
            curr_state = (fft_pow(t) > power_30) + (fft_pow(t) > power_70);
            if curr_state < 1
                had_low = 1;
                had_high = 0;
            elseif curr_state > 1
                had_high = 1;
                if had_low
                    had_low = 0;
                    features(40, w) = features(40, w) + 1;
                    peak_loc(features(40, w)) = t;
                end
            end
        end        

        if features(40, w) > 1        
            features(41, w) = mean(peak_loc(2:end)-peak_loc(1:end-1));
            features(41, w) = mean(abs((peak_loc(2:end)-peak_loc(1:end-1))-features(41, w)));        
        else
            features(41, w) = length(fft_pow)/4;
        end
        features(42, w) = max_power-min_power;
        features(43, w) = mean(fft_pow);

        % coarser resolution frequencies
        for f = 44:47
            i1 = (f-44)*4+1;
            i2 = i1+3;
            features(f, w) = mean(coarse_fft_data(f-43, :));
            features(f+4, w) = std(coarse_fft_data(f-43, :));
        end   
        
        features(52:138, w) = compute_local_features_window(fft_data, fft_pow);    
        
    end
        
    
    
    